/**	\file ComplexType.h
 **	\date  2008-05-26
 **	\author grymse@alhem.net
**/
/*
Copyright (C) 2008-2009  Anders Hedstrom

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#ifndef _ComplexType_H
#define _ComplexType_H

#include <string>
#include <list>
#include <map>
#include "TypeBase.h"


class Wsdl;
class XmlNode;

class ComplexType : public TypeBase
{
public:
	ComplexType(const Wsdl& wsdl, const XmlNode& node, const std::string& name = "", const std::string& ns_prefix = "");
	~ComplexType();

	const std::string& Name() const { return m_name; }
	const std::string Documentation() const;

	int NumberOfMembers() const;
	bool GetRef(std::string& ref) const;

	std::list<std::pair<std::string, XmlNode> > GetMembers() const;
	std::list<std::pair<std::string, XmlNode> > GetAttributes() const;

	void CreateInterface(FILE *fil, int lvl = 1) const;
	void CreateImplementation(FILE *fil) const;

	const std::string CName() const;

	bool OkToGenerate() const;
	void SetGenerated(bool x = true);
	bool IsGenerated() const;

	bool IsSequence() const { return m_b_is_sequence; }
	bool IsAll() const { return m_b_is_all; }
	bool IsComplexContent() const { return m_b_complexContent; }
	bool IsArray() const { return m_b_is_array; }
	bool IsMixed() const { return m_b_is_mixed; }

	static void ShowGenerated();

private:
	void reset_members(std::list<std::pair<std::string, XmlNode> >& members, std::list<std::pair<std::string, XmlNode> >& attrs, FILE *fil) const;
	void clear_members(std::list<std::pair<std::string, XmlNode> >& members, std::list<std::pair<std::string, XmlNode> >& attrs, FILE *fil) const;
	void from_xml(std::list<std::pair<std::string, XmlNode> >& members, std::list<std::pair<std::string, XmlNode> >& attrs, FILE *fil) const;
	void to_string(std::list<std::pair<std::string, XmlNode> >& members, std::list<std::pair<std::string, XmlNode> >& attrs, FILE *fil) const;
	XmlNode Start() const;
	//
	std::string m_name;
	bool m_b_is_sequence;
	bool m_b_is_all;
	bool m_b_complexContent;
	bool m_b_is_array;
	bool m_b_is_mixed;
	std::string m_extends;
	std::string m_extends_cls;
static	std::map<std::string, bool> m_b_generated;
};




#endif // _ComplexType_H
